@@ -1,12 +1,24 @@
 package Date::Baha::i;
 
 use strict;
-#use base qw(Exporter);
-#use vars qw($@EXPORT @EXPORT_OK);
-#@EXPORT = @EXPORT_OK = qw(date);
-use vars qw($VERSION); $VERSION = '0.03.1';
+use vars qw($VERSION); $VERSION = '0.04.1';
+use base qw(Exporter);
+use vars qw(@EXPORT @EXPORT_OK);
+@EXPORT = @EXPORT_OK = qw(
+    cycles
+    date
+    day_of_week
+    days
+    days_of_the_week
+    greg_to_bahai
+    holy_days
+    months
+    years
+);
+
 use Date::Calc qw(
     Add_Delta_Days
+    Date_to_Time
     Day_of_Week
     Delta_Days
     Timezone
@@ -18,7 +30,7 @@ use constant MARCH => 3;
 use constant LAST_START_DAY => 2;   # First day of the fast.
 use constant YEAR_START_DAY => 21;  # Spring equinox.
 use constant FEBRUARY => 2;
-use constant LEAP_START_DAY => 25;  # The intercalary days.
+use constant LEAP_START_DAY => 26;  # The intercalary days.
 use constant FIRST_YEAR  => 1844;
 use constant ADJUST_YEAR => 1900;
 use constant CYCLE_YEAR => qw(
@@ -142,13 +154,13 @@ sub date {
             $month, $day, MARCH, LAST_START_DAY, MARCH, YEAR_START_DAY - 1
          ))
     ) { 
-        $bahai_day = delta_month_days (MARCH, 1, $year, $month, $day);
+        $bahai_day = delta_month_days (MARCH, LAST_START_DAY - 1, $year, $month, $day);
         $bahai_month = FACTOR - 1;
     }   
 
     # If we still didn't find a month, it is Ayyam-i-Ha!
     unless ($found) {
-        $bahai_day = delta_month_days (FEBRUARY, LEAP_START_DAY, $year, $month, $day);
+        $bahai_day = delta_month_days (FEBRUARY, LEAP_START_DAY - 1, $year, $month, $day);
         $bahai_month = -1;
     }
 
@@ -157,6 +169,15 @@ sub date {
 }
 # }}}
 
+# greg_to_bahai function {{{
+sub greg_to_bahai {
+    my ($y, $m, $d) = @_;
+    # It would seem that Date::Calc::Date_to_Time (and Time_to_Date)
+    # is broken wrt the day.  "+ 1"?  WTF?
+    return date (Date_to_Time ($y, $m, $d + 1, 0, 0, 0));
+}
+# }}}
+
 # Helper functions {{{
 # The Baha'i week starts on Saturday.
 sub day_of_week {
@@ -243,11 +264,22 @@ __END__
 
 =head1 NAME
 
-Date::Baha'i - Compute the numeric and named Baha'i date.
+Date::Baha::i - Compute the numeric and named Baha'i date.
 
 =head1 SYNOPSIS
 
-  %bahai_date = Date::Baha'i::date ();
+  use Date::Baha'i;
+
+  %bahai_date = date ();
+
+  %bahai_date = greg_to_bahai ($year, $month, $day);
+
+  @ret = cycles ();
+  @ret = years ();
+  @ret = months ();
+  @ret = days ();
+  @ret = days_of_the_week ();
+  @ret = holy_days ();
 
 =head1 ABSTRACT
 
@@ -306,25 +338,25 @@ Each cycle of nineteen years is called a Vahid. Nineteen cycles constitute a per
 
 The names of the years in each cycle are: 
 
-  1.  Alif - The Letter "A"
-  2.  Ba - The letter "B"
-  3.  Ab - Father
-  4.  Dal - The letter "D"
-  5.  Bab - Gate
-  6.  Vav - The letter "V"
-  7.  Abad - Eternity
-  8.  Jad - Generosity
-  9.  Baha - Splendour
-  10. Hubb - Love
+  1.  Alif   - The Letter "A"
+  2.  Ba     - The letter "B"
+  3.  Ab     - Father
+  4.  Dal    - The letter "D"
+  5.  Bab    - Gate
+  6.  Vav    - The letter "V"
+  7.  Abad   - Eternity
+  8.  Jad    - Generosity
+  9.  Baha   - Splendour
+  10. Hubb   - Love
   11. Bahhaj - Delightful
-  12. Javab - Answer
-  13. Ahad - Single
+  12. Javab  - Answer
+  13. Ahad   - Single
   14. Vahhab - Bountiful
-  15. Vidad - Affection
-  16. Badi - Beginning
-  17. Bahi - Luminous
-  18. Abha - Most Luminous
-  19. Vahid - Unity
+  15. Vidad  - Affection
+  16. Badi   - Beginning
+  17. Bahi   - Luminous
+  18. Abha   - Most Luminous
+  19. Vahid  - Unity
 
 There are eleven Holy Days which Baha'is celebrate. On [many] of these days, all work should cease. They are listed in chronological order according to the Baha'i calendar.
 
@@ -358,7 +390,7 @@ Commemorates the date in 1819 when the Bab was born in Shiraz, Iran
 
 * Birth of Baha'u'llah - 12 November
 
-Commemorates the date in 1817 when Bahá'u'lláh was born in Tihran, Iran
+Commemorates the date in 1817 when Baha'u'llah was born in Tihran, Iran
 
 - Work does not have to cease on these Holy Days:
 
@@ -402,9 +434,9 @@ L<http://www.moonwise.co.uk/year/159bahai.htm>
 
 =head2 date
 
-  %bahai_date = Date::Baha::i::date ()
+  %bahai_date = date ([time])
 
-This function returns a hash of the date names and numbers from a system time() stamp.
+This function returns a hash of the Baha'i date names and numbers from a user provided system time() stamp.
 
 The hash returned has these keys:
 
@@ -422,24 +454,48 @@ The hash returned has these keys:
   dow_name
   timezone
 
+=head2 greg_to_bahai
+
+  %bahai_date = greg_to_bahai ($year, $month, $day);
+
+Compute the Baha'i date from a Gregorian year, month, day triple.
+
 =head2 cycles
 
+  @cycles = cycles ();
+
 Return the 19 cycle names as an array.
 
 =head2 years
 
+  @years = years ();
+
 Return the 19 year names as an array.
 
 =head2 months
 
-Return the 19 month names as an array, along with  the intercalary days ("Ayyam-i-Ha") as the last element.
+  @months = months ();
+
+Return the 19 month names as an array, along with the intercalary days ("Ayyam-i-Ha") as the last element.
 
 =head2 days
 
+  @days = days ();
+
 Return the 19 day names as an array.
 
+=head2 days_of_the_week
+
+  @days = days_of_the_week ();
+
+Return the 7 day-of-the-week names as an array.
+
+Return the 
+
 =head2 holy_days
 
+  @days = holy_days ();
+
 Return the holy days as a hash where the keys are the holy day names and the values are array references.  These array references are composed of two or three elements, where the first is the month, the second is the day, and the third is the (optional) number of days observed.  These dates are currently in standard (non-Baha'i) format.
 
 =head1 DEPENDENCIES
@@ -450,16 +506,12 @@ L<Date::Calc>
 
 Overload localtime() and gmtime?
 
-Convert between Julian dates/Unix timestamps and Baha'i dates.
+Convert between Gregorian dates/Unix timestamps and Baha'i dates.
 
 Base the date computation on the time of day (the Baha'i day begins at Sunset).
 
 Output unicode.
 
-=head1 HISTORY
-
-See the Changes file in this distribution.
-
 =head1 DEDICATION
 
 Hi Kirsten  : )
